/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.binding.http;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.component.binding.BindingFaultReference;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceAction.Direction;
import org.bluestemsoftware.specification.eoa.component.policy.rt.ElementPolicy;

public final class HTTPBindingFaultReference extends BindingFaultReference {

    /**
     * Constructs a protocol specific, i.e. SOAP, <code>BindingFaultReference</code>
     * component.
     * 
     * @param faultName
     *        Qualified name of referenced <code>BindingFault</code>.
     * @param direction
     *        defined on abstract action to which this concrete action is bound.
     * @param elementPolicy
     *        Contains policy expressions which may be attached to
     *        <code>BindingFaultReference</code>.
     */

    public HTTPBindingFaultReference(QName faultName, Direction direction, ElementPolicy elementPolicy) {
        super(faultName, direction, elementPolicy);
    }

    // *********************** Identity Methods ************************************

    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("direction: ");
        sb.append(System.getProperty("line.separator"));
        sb.append(direction);
        sb.append(System.getProperty("line.separator"));
        sb.append("faultName: ");
        sb.append(faultName);
        return sb.toString();
    }

    @Override
    public int hashCode() {
        return hashCode(direction) + hashCode(faultName);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }

        if (!(obj instanceof HTTPBindingFaultReference)) {
            return false;
        }

        if ((faultName == ((HTTPBindingFaultReference)obj).faultName || (faultName != null && faultName
                .equals(((HTTPBindingFaultReference)obj).faultName)))
                && (direction == ((HTTPBindingFaultReference)obj).direction || (direction != null && direction
                        .equals(((HTTPBindingFaultReference)obj).direction)))) {
            return true;
        }

        return false;
    }

    private int hashCode(Object object) {
        return (object == null ? 0 : object.hashCode());
    }

}
