/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.binding.soap;

import org.bluestemsoftware.specification.eoa.component.binding.BindingMessageReference;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceAction.Direction;
import org.bluestemsoftware.specification.eoa.component.policy.rt.ElementPolicy;

public final class SOAPBindingMessageReference extends BindingMessageReference {

    private SOAPMessage soapMessage;

    /**
     * Constructs a protocol specific, i.e. SOAP, <code>BindingMessageReference</code>
     * component.
     * 
     * @param direction
     *        defined on abstract action to which this concrete action is bound.
     * @param elementPolicy
     *        Contains policy expressions which may be attached to
     *        <code>BindingMessageReference</code>.
     * @param soapMessage
     *        metadata which abstractly describes concrete message format
     */
    public SOAPBindingMessageReference(Direction direction, ElementPolicy elementPolicy, SOAPMessage soapMessage) {
        super(direction, elementPolicy);
        if (soapMessage == null) {
            throw new IllegalArgumentException("soapMessage null");
        } else {
            this.soapMessage = soapMessage;
        }
        soapMessage.setParent(this);
    }

    /**
     * Gets metadata which abstractly describes concrete message format.
     * 
     * @return SOAPMessage
     */
    public SOAPMessage getSoapMessage() {
        return soapMessage;
    }
    
    // *********************** Identity Methods ************************************

    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("direction: ");
        sb.append(System.getProperty("line.separator"));
        sb.append(direction);
        sb.append(System.getProperty("line.separator"));
        sb.append("soapMessage: ");
        sb.append(soapMessage);
        return sb.toString();
    }

    @Override
    public int hashCode() {
        return hashCode(direction) + hashCode(soapMessage);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }

        if (!(obj instanceof SOAPBindingMessageReference)) {
            return false;
        }

        if (
                (soapMessage == ((SOAPBindingMessageReference)obj).soapMessage || (soapMessage != null && soapMessage.equals(
                        ((SOAPBindingMessageReference)obj).soapMessage)))
                        &&
                (direction == ((SOAPBindingMessageReference)obj).direction || (direction != null && direction.equals(
                ((SOAPBindingMessageReference)obj).direction)))
        ) {
            return true;
        }

        return false;
    }

    private int hashCode(Object object) {
        return (object == null ? 0 : object.hashCode());
    }

}
