/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.connector.jms;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;

import org.bluestemsoftware.specification.eoa.system.container.Connector;

public class JMSConnector extends Connector implements ConnectionFactory {

    public static final String TYPE = JMSConnector.class.getName();

    /*
     * (non-Javadoc)
     * 
     * SPI
     */
    public interface Provider extends Connector.Provider, ConnectionFactory {
    }

    public JMSConnector(JMSConnectorFactory factory, Connector.Provider provider, String name) {
        super(factory, provider, name);
    }

    /** *************************** ConnectionFactory Impl ************************** * */

    /*
     * (non-Javadoc)
     * @see javax.jms.ConnectionFactory#createConnection()
     */
    public Connection createConnection() throws JMSException {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            return ((Provider)provider).createConnection();
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

    /*
     * (non-Javadoc)
     * @see javax.jms.ConnectionFactory#createConnection(java.lang.String, java.lang.String)
     */
    public Connection createConnection(String userName, String password) throws JMSException {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            return ((Provider)provider).createConnection(userName, password);
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

}
