/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.feature.naming;

import org.bluestemsoftware.specification.eoa.ext.container.standalone10.StandaloneContainer;
import org.bluestemsoftware.specification.eoa.ext.feature.http.server.HTTPServerFeature;
import org.bluestemsoftware.specification.eoa.system.container.Connector;
import org.bluestemsoftware.specification.eoa.system.container.Container;
import org.bluestemsoftware.specification.eoa.system.server.Feature;
import org.bluestemsoftware.specification.eoa.system.server.Server;

/**
 * Provides a {@link Server} wide naming context which MAY be used by applications to access
 * EOA resources using the JNDI api.
 * <p>
 * A compliant implementation MUST bind all {@link Connector} instances retrieved via
 * {@link Container#getConnectors()} within the "eoa:connector" namespace using each respective
 * connector's name, e.g. "eoa:connector/SystemDataSource". A compliant implementation MUST
 * bind the transaction object retrieved via {@link StandaloneContainer#getUserTransaction()}
 * to the name "eoa:javax/UserTransaction".
 * <p>
 * Note that the "java:comp" namespace, which is a "virtual" naming context specific to each
 * java component instance, e.g. a servlet, ejb, or jsp, is not managed by this feature. It MAY
 * instead be managed by the {@link HTTPServerFeature}.
 */
public class NamingContextFeature extends Feature {

    public static final String TYPE = NamingContextFeature.class.getName();

    /*
     * (non-Javadoc)
     * 
     * SPI
     */
    public interface Provider extends Feature.Provider {
        // no implementation. functionality accessed via JNDI
    }

    public NamingContextFeature(NamingContextFeatureFactory factory, Feature.Provider provider) {
        super(factory, provider);
    }

}
