/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.feature.ws.addressing.soap.fault;

import java.util.Arrays;
import java.util.List;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.component.message.rt.Content;
import org.bluestemsoftware.specification.eoa.component.message.rt.ContentFactory;
import org.bluestemsoftware.specification.eoa.ext.binding.soap.SOAPVersion;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.addressing.WSA;
import org.w3c.dom.Element;

public class ActionMismatchFault extends InvalidHeaderFault {
    
    private static final long serialVersionUID = 1L;
    
    private String action;
    private String soapAction;

    public ActionMismatchFault(SOAPVersion soapVersion, String namespace, String action, String soapAction) {
        super(soapVersion, namespace, WSA.ACTION);
        if (action == null) {
            throw new IllegalArgumentException("action null");
        }
        this.action = action;
        if (soapAction == null) {
            throw new IllegalArgumentException("soapAction null");
        }
        this.soapAction = soapAction;
    }
    
    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.binding.soap.rt.SOAPFault#getFaultSubCodes()
     */
    @Override
    public List<QName> getFaultSubCodes() {
        return Arrays.asList(faultName, new QName(namespace, "ActionMismatch"));
    }
    
   /*
    * (non-Javadoc)
    * @see org.bluestemsoftware.specification.eoa.ext.feature.ws.addressing.soap.fault.InvalidHeaderFault#appendFaultDetail(org.bluestemsoftware.specification.eoa.component.message.rt.Content)
    */
    @Override
    public void appendFaultDetail(Content faultEnvelope) {
        
        super.appendFaultDetail(faultEnvelope);
        
        ContentFactory factory = faultEnvelope.getFactory();
        String qualifiedElementName = WSA.NAMESPACE_PREFIX + ":" + "ProblemAction";
        Element problemActionElement = factory.createElementNS(namespace, qualifiedElementName);
        qualifiedElementName = WSA.NAMESPACE_PREFIX + ":" + "Action";
        Element actionElement = factory.createElementNS(namespace, qualifiedElementName);
        problemActionElement.appendChild(actionElement);
        actionElement.appendChild(factory.createTextNode(action));
        qualifiedElementName = WSA.NAMESPACE_PREFIX + ":" + "SoapAction";
        Element soapActionElement = factory.createElementNS(namespace, qualifiedElementName);
        problemActionElement.appendChild(soapActionElement);
        soapActionElement.appendChild(factory.createTextNode(soapAction));
        
        // if we're writing to a soap 1.1. envelope, we must set
        // detail within header (wrapper element already added by
        // super), i.e. because fault not related to body of request.
        // else we set on detail element
        
        if (soapVersion == SOAPVersion.SOAP11) {
            Element headerElement = getFaultHeaderElement(faultEnvelope);
            QName faultDetailElement = new QName(namespace, "FaultDetail");
            Element parent = getChildElement(headerElement, faultDetailElement);
            parent.appendChild(problemActionElement);
        } else {
            Element faultDetailElement = getFaultDetailElement(faultEnvelope);
            faultDetailElement.appendChild(problemActionElement);
        }
        
    }

}
