/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.feature.ws.addressing.soap.fault;

import java.util.Arrays;
import java.util.List;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.component.message.rt.Content;
import org.bluestemsoftware.specification.eoa.component.message.rt.ContentFactory;
import org.bluestemsoftware.specification.eoa.ext.binding.soap.SOAPVersion;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.addressing.WSA;
import org.w3c.dom.Element;

public class ActionNotSupportedFault extends WSAddressingFault {
    
    private static final long serialVersionUID = 1L;
    
    public static final String REASON = "The action cannot be processed at the receiver";
    
    private String namespace;
    private String action;

    public ActionNotSupportedFault(SOAPVersion soapVersion, String namespace, String action) {
        super(soapVersion, new QName(namespace, "ActionNotSupported", WSA.NAMESPACE_PREFIX), REASON);
        if (action == null) {
            throw new IllegalArgumentException("action null");
        }
        this.namespace = namespace;
        this.action = action;
    }
    
    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.binding.soap.rt.SOAPFault#getFaultSubCodes()
     */
    @Override
    public List<QName> getFaultSubCodes() {
        return Arrays.asList(faultName);
    }
    
    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.binding.soap.rt.SOAPSenderFault#appendFaultDetail(org.bluestemsoftware.specification.eoa.component.message.rt.Content)
     */
    @Override
    public void appendFaultDetail(Content faultEnvelope) {
        
        ContentFactory factory = faultEnvelope.getFactory();
        String qualifiedElementName = WSA.NAMESPACE_PREFIX + ":" + "ProblemAction";
        Element problemActionElement = factory.createElementNS(namespace, qualifiedElementName);
        qualifiedElementName = WSA.NAMESPACE_PREFIX + ":" + "Action";
        Element actionElement = factory.createElementNS(namespace, qualifiedElementName);
        problemActionElement.appendChild(actionElement);
        actionElement.appendChild(factory.createTextNode(action));
        
        // if we're writing to a soap 1.1. envelope, we must set
        // detail within header, i.e. because fault not related
        // to body of request. else we set on detail element
        
        if (soapVersion == SOAPVersion.SOAP11) {
            Element headerElement = getFaultHeaderElement(faultEnvelope);
            qualifiedElementName = WSA.NAMESPACE_PREFIX + ":FaultDetail";
            Element faultDetailElement = factory.createElementNS(namespace, qualifiedElementName);
            faultDetailElement.appendChild(problemActionElement);
            headerElement.appendChild(faultDetailElement);
        } else {
            Element faultDetailElement = getFaultDetailElement(faultEnvelope);
            faultDetailElement.appendChild(problemActionElement);
        }
                
    }

}
