/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.feature.ws.addressing.soap.fault;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.component.message.rt.Content;
import org.bluestemsoftware.specification.eoa.component.message.rt.ContentFactory;
import org.bluestemsoftware.specification.eoa.ext.binding.soap.SOAPVersion;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.addressing.WSA;
import org.w3c.dom.Element;

public class InvalidHeaderFault extends WSAddressingFault {

    private static final long serialVersionUID = 1L;

    public static final String REASON = "A header representing a Message Addressing Property is not valid and the message cannot be processed";
    
    protected String namespace;
    protected String problemHeader;

    public InvalidHeaderFault(SOAPVersion soapVersion, String namespace, String problemHeader) {
        super(soapVersion, new QName(namespace, "InvalidAddressingHeader", WSA.NAMESPACE_PREFIX), REASON);
        if (namespace == null) {
            throw new IllegalArgumentException("namespace null");
        } else {
            this.namespace = namespace;
        }
        if (problemHeader == null) {
            throw new IllegalArgumentException("problemHeader null");
        } else {
            this.problemHeader = problemHeader;
        }
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.binding.soap.rt.SOAPSenderFault#appendFaultDetail(org.bluestemsoftware.specification.eoa.component.message.rt.Content)
     */
    @Override
    public void appendFaultDetail(Content faultEnvelope) {
        
        ContentFactory factory = faultEnvelope.getFactory();
        String qualifiedElementName = WSA.NAMESPACE_PREFIX + ":ProblemHeaderQName";
        Element problemHeaderElement = factory.createElementNS(namespace, qualifiedElementName);
        String qualifiedHeaderName = WSA.NAMESPACE_PREFIX + ":" + problemHeader;
        problemHeaderElement.appendChild(factory.createTextNode(qualifiedHeaderName));
        
        // if we're writing to a soap 1.1. envelope, we must set
        // detail within header, i.e. because fault not related
        // to body of request. else we set on detail element
        
        if (soapVersion == SOAPVersion.SOAP11) {
            Element headerElement = getFaultHeaderElement(faultEnvelope);
            qualifiedElementName = WSA.NAMESPACE_PREFIX + ":FaultDetail";
            Element faultDetailElement = factory.createElementNS(namespace, qualifiedElementName);
            faultDetailElement.appendChild(problemHeaderElement);
            headerElement.appendChild(faultDetailElement);
        } else {
            Element faultDetailElement = getFaultDetailElement(faultEnvelope);
            faultDetailElement.appendChild(problemHeaderElement);
        }
                
    }
    
}
