/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.feature.ws.transport.http;

import org.bluestemsoftware.specification.eoa.component.binding.rt.BindingRT;
import org.bluestemsoftware.specification.eoa.component.engine.rt.TransportModule;
import org.bluestemsoftware.specification.eoa.ext.feature.http.client.HTTPClientRequest;
import org.bluestemsoftware.specification.eoa.ext.feature.http.client.HTTPClientResponse;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.transport.rt.TransportFault;

public class HTTPTransportModule extends TransportModule {

    public static final String TYPE = HTTPTransportModule.class.getName();

    /*
     * (non-Javadoc)
     * 
     * SPI
     */
    public interface Provider extends TransportModule.Provider {
        public HTTPClientResponse spi_doSend(HTTPClientRequest request, boolean handleRedirects) throws TransportFault;
    }

    public HTTPTransportModule(HTTPTransportModule.Provider provider, HTTPTransportFeature feature) {
        super(provider, feature);
    }

    /**
     * Sends an <code>HTTPClientRequest</code> and returns an <code>HTTPClientResponse</code>.
     * Invoked by an instance of {@link BindingRT} which utilizes HTTP to send/receive
     * messages.
     * 
     * @param request
     *        http request.
     * @param handleRedirects
     *        If true, response status codes in the range 3xx MUST be handled by a compliant
     *        module implementation. If after an unspecified number of attempts, the send
     *        fails, a transport fault MUST be thrown. If false, the first encountered 3xx code
     *        MUST be returned within the response object.
     * @return Response http response.
     * @throws TransportFault
     *         if an error occurs sending request.
     */
    public HTTPClientResponse doSend(HTTPClientRequest request, boolean handleRedirects) throws TransportFault {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            return ((Provider)provider).spi_doSend(request, handleRedirects);
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

}
