/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.feature.ws.transport.http;

import java.util.Collections;
import java.util.Map;
import java.util.Set;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.component.engine.rt.FeatureModule.WSSpecification;
import org.bluestemsoftware.specification.eoa.ext.feature.auth.HostInfo;

public abstract class WSHTTP implements WSSpecification {

    public static WSHTTP getInstance(String namespace) {
        if (namespace.equals(WSHTTP10.Policy.PVN)) {
            return new WSHTTP10();
        } else {
            return null;
        }
    }

    public interface Policy extends WSSpecification.Policy {
        public interface Infoset extends WSSpecification.Infoset {
        }
    }

    public static class WSHTTP10 extends WSHTTP {

        public static final String NAMESPACE_URI = "http://bluestemsoftware.org/specification/eoa/ext/feature/ws/transport/http/1.0";

        public static class Infoset implements WSSpecification.Infoset {
        }

        public static class Policy implements WSHTTP.Policy {

            public static final String PVN = "http://bluestemsoftware.org/specification/eoa/ext/feature/ws/transport/http/policy/1.0";
            public static final String PVN_PREFIX = "wshttp";

            private Map<HostInfo, AuthInfo> authInfos;

            public Policy(Map<HostInfo, AuthInfo> authInfos) {
                this.authInfos = authInfos;
            }

            public static class Infoset implements WSHTTP.Policy.Infoset {
                public static final QName SCHEME_ELEM = new QName(PVN, "scheme", PVN_PREFIX);
                public static final QName REALM_ELEM = new QName(PVN, "realm", PVN_PREFIX);
                public static final QName USER_ELEM = new QName(PVN, "user", PVN_PREFIX);
                public static final QName NONE_ELEM = new QName(PVN, "none", PVN_PREFIX);
            }

            public Infoset getInfoset() {
                return new Infoset() {
                };
            }

            /**
             * Gets <code>AuthInfo</code> objects, each of which is mapped to a host info object.
             * @return
             */
            public Map<HostInfo, AuthInfo> getAuthInfos() {
                return authInfos;
            }

            public static class AuthInfo {

                private String role;
                private HTTPAuthenticationScheme scheme;
                private String user;

                public AuthInfo(String role, HTTPAuthenticationScheme scheme, String user) {
                    this.role = role;
                    this.scheme = scheme;
                    this.user = user;
                }

                public String getRole() {
                    return role;
                }

                public HTTPAuthenticationScheme getScheme() {
                    return scheme;
                }

                public String getUser() {
                    return user;
                }
            }

        }

        public WSHTTP10() {
        }

        public WSHTTP10(Set<WSSpecification.Policy> alternatives) {
            this.alternatives = alternatives;
        }

        private Set<WSSpecification.Policy> alternatives;

        @Override
        public WSSpecification.Infoset getInfoSet() {
            return new Infoset() {
            };
        }

        @Override
        public String getNamespace() {
            return NAMESPACE_URI;
        }

        @Override
        @SuppressWarnings("unchecked")
        public Set<WSSpecification.Policy> getPolicy() {
            if (alternatives == null) {
                return Collections.EMPTY_SET;
            }
            return alternatives;
        }

    }

    public abstract Infoset getInfoSet();

    public abstract String getNamespace();

    public abstract Set<WSSpecification.Policy> getPolicy();

}
