/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.feature.ws.transport.vm;

import org.bluestemsoftware.specification.eoa.component.engine.rt.TransportModule;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.transport.rt.TransportFault;

public class VMTransportModule extends TransportModule {

    public static final String TYPE = VMTransportModule.class.getName();

    /*
     * (non-Javadoc)
     * 
     * SPI
     */
    public interface Provider extends TransportModule.Provider {
        public void spi_doSend(VMTransportRequest request) throws TransportFault;
    }

    public VMTransportModule(VMTransportModule.Provider provider, VMTransportFeature feature) {
        super(provider, feature);
    }

    /**
     * Sends a concrete message instance to a locally defined Endpoint.
     * 
     * @param request
     *        An addressed concrete message instance, i.e. formatted according to a specific
     *        messaging protocol, e.g. a SOAP envelope.
     * @throws VMTransportFeatureException
     *         if an error occurs processing message.
     */
    public void doSend(VMTransportRequest request) throws TransportFault {
        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factory.getFactoryContext().getClassLoader());
            ((Provider)provider).spi_doSend(request);
        } finally {
            thread.setContextClassLoader(cl);
        }
    }

}
