/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20;

import java.util.Map;
import java.util.Set;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.ApplicationElement;
import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20.ext.EngineElement;
import org.w3c.dom.Element;

/**
 * Models root element of a <code>WSDL20Document</code>. Note that it is implementation's
 * responsibility to switch thread context classloader as required to/from
 * {@link WSDL20Factory} deployment classloader.
 */

public interface DescriptionElement extends Element {

    public static final QName NAME = new QName(WSDL20Factory.WSDL_NS, "description", WSDL20Factory.WSDL_NS_PREFIX);

    public void setTargetNamespace(String targetNamespace);

    public String getTargetNamespace();

    public TypesElement getTypesElement();

    public ImportElement addImportElement(ImportElement importElement);

    /**
     * Gets import elements.
     * @param namespace
     *        imported
     * @return Set of import elements which import indicated namespace or an empty set if none
     *         defined.
     */
    public Set<ImportElement> getImportElements(String namespace);

    public IncludeElement addIncludeElement(IncludeElement includeElement);

    /**
     * Gets include elements.
     * @return Set of include elements or an empty set if none defined.
     */
    public Set<IncludeElement> getIncludeElements();

    public TypesElement setTypesElement(TypesElement typesElement);

    public InterfaceElement addInterfaceElement(InterfaceElement interfaceElement);

    /**
     * Gets interface elements.
     * @return Map of elements or an empty map if none defined.
     */
    public Map<String, InterfaceElement> getInterfaceElements();

    public BindingElement addBindingElement(BindingElement bindingElement);

    /**
     * Gets binding elements.
     * @return Map of elements or an empty map if none defined.
     */
    public Map<String, BindingElement> getBindingElements();

    public ServiceElement addServiceElement(ServiceElement serviceElement);

    /**
     * Gets service elements.
     * @return Map of elements or an empty map if none defined.
     */
    public Map<String, ServiceElement> getServiceElements();

    public ApplicationElement addApplicationElement(ApplicationElement applicationElement);

    /**
     * Gets application elements.
     * @return Map of elements or an empty map if none defined.
     */
    public Map<String, ApplicationElement> getApplicationElements();

    public EngineElement addEngineElement(EngineElement engineElement);

    /**
     * Gets engine elements.
     * @return Map of elements or an empty map if none defined.
     */
    public Map<String, EngineElement> getEngineElements();

}
