/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.idl.wsdl.wsdl20;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.ext.idl.wsdl.WSDLException;
import org.w3c.dom.Element;

/**
 * Models a <code>WSDL</code> 2.0 'import' element. Note that it is implementation's
 * responsibility to switch thread context classloader as required to/from
 * {@link WSDL20Factory} deployment classloader.
 */

public interface ImportElement extends Element {

    public static final QName NAME = new QName(WSDL20Factory.WSDL_NS, "import", WSDL20Factory.WSDL_NS_PREFIX);

    public void setNamespace(String namespace);

    public String getNamespace();

    public void setLocation(String location);

    public String getLocation();

    /**
     * Gets imported document. If location not yet resolved, impl MUST attempt to resolve it.
     * Resolution SHOULD be deferred until requested, i.e. which could be never.
     * 
     * @return WSDLDocument referenced document or <code>null</code> if location attribute
     *         not specified, i.e. in which case it is assumed that referenced component(s) is
     *         defined within a separate deployment.
     * @throws WSDLException
     *         If location attribute is specified and an error occurs attempting to dereference
     *         it.
     */
    public WSDL20Document getDocument() throws WSDLException;

}
