/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the Ext EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 * 
 * [1] http://alakai.org/licensing/specification-eoa-ext-license.html
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.schema.xs.xs10;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStreamWriter;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.xml.XMLConstants;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXSource;

import org.bluestemsoftware.specification.eoa.DeploymentException;
import org.bluestemsoftware.specification.eoa.component.ComponentContext;
import org.bluestemsoftware.specification.eoa.component.ComponentDeployment;
import org.bluestemsoftware.specification.eoa.component.FragmentIdentifier;
import org.bluestemsoftware.specification.eoa.component.FragmentIdentifier.Scheme;
import org.bluestemsoftware.specification.eoa.component.RootComponent.ComponentName;
import org.bluestemsoftware.specification.eoa.component.RootComponent.ComponentType;
import org.bluestemsoftware.specification.eoa.component.RootComponent.RuntimeProvidable;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceMessageReference;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceOperation.Style;
import org.bluestemsoftware.specification.eoa.component.message.InterfaceMessage;
import org.bluestemsoftware.specification.eoa.component.message.MessagePart;
import org.bluestemsoftware.specification.eoa.component.policy.rt.ElementPolicy;
import org.bluestemsoftware.specification.eoa.component.types.ElementDeclaration;
import org.bluestemsoftware.specification.eoa.component.types.FragmentExpression;
import org.bluestemsoftware.specification.eoa.component.types.Schema;
import org.bluestemsoftware.specification.eoa.component.types.SchemaComponent;
import org.bluestemsoftware.specification.eoa.component.types.SchemaFragment;
import org.bluestemsoftware.specification.eoa.component.types.SchemaImport;
import org.bluestemsoftware.specification.eoa.component.types.SchemaInclude;
import org.bluestemsoftware.specification.eoa.component.types.SchemaProperty;
import org.bluestemsoftware.specification.eoa.component.types.TypeDefinition;
import org.bluestemsoftware.specification.eoa.component.types.rt.ExpressionRT;
import org.bluestemsoftware.specification.eoa.component.types.rt.SchemaDocument;
import org.bluestemsoftware.specification.eoa.component.types.rt.SchemaException;
import org.bluestemsoftware.specification.eoa.ext.ExtensionFactory;
import org.bluestemsoftware.specification.eoa.ext.expression.ExpressionException;
import org.bluestemsoftware.specification.eoa.ext.expression.ExpressionFactory;
import org.bluestemsoftware.specification.eoa.ext.expression.xpath.xpath10.XPathExpression;
import org.bluestemsoftware.specification.eoa.ext.idl.IDLDocument;
import org.bluestemsoftware.specification.eoa.ext.schema.SchemaFactory;
import org.bluestemsoftware.specification.eoa.ext.schema.xs.XMLSchemaException;
import org.bluestemsoftware.specification.eoa.system.SystemContext;
import org.w3c.dom.Attr;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMConfiguration;
import org.w3c.dom.DOMException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.EntityReference;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;
import org.w3c.dom.UserDataHandler;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSOutput;
import org.w3c.dom.ls.LSSerializer;
import org.xml.sax.ContentHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.ext.LexicalHandler;

/**
 * A factory for producing XML Schema documents which conform to version 1.0 of the XML Schema
 * Language specification.
 */
public class XMLSchemaFactory extends SchemaFactory {

    public static final QName QUERY_ELEMENT = new QName(EOA_SCHEMA_EXT_NS, "query");
    public static final QName MESSAGE_ELEMENT = new QName(EOA_SCHEMA_EXT_NS, "message");

    /*
     * (non-Javadoc)
     * 
     * SPI
     */
    public interface Provider extends SchemaFactory.Provider {

        public InputSource spi_dereferenceLocation(EntityResolver entityResolver, String location) throws XMLSchemaException;

        public Set<SchemaDocument> spi_readSchemas(EntityResolver entityResolver, Source source) throws XMLSchemaException;

    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.ExtensionFactory#getExtensionType()
     */
    public String getExtensionType() {
        return XMLSchema.TYPE;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.schema.SchemaFactory#readSchemas(org.xml.sax.EntityResolver,
     *      javax.xml.transform.Source)
     */
    @Override
    public Set<SchemaDocument> readSchemas(EntityResolver entityResolver, Source source) throws XMLSchemaException {

        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        try {
            thread.setContextClassLoader(factoryContext.getClassLoader());
            return ((Provider)provider).spi_readSchemas(entityResolver, source);
        } finally {
            thread.setContextClassLoader(cl);
        }

    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.ComponentReader#readComponents(org.bluestemsoftware.specification.eoa.component.ComponentContext, org.bluestemsoftware.specification.eoa.ext.idl.IDLDocument)
     */
    public void readComponents(ComponentContext componentContext, IDLDocument idlDocument) throws DeploymentException {

        ComponentDeployment deployment = componentContext.getDeployment();

        for (Document grammarDocument : idlDocument.getGrammarDocuments()) {
            
            Element schemaElement = grammarDocument.getDocumentElement();
            
            if (schemaElement == null) {
                throw new DeploymentException("Grammar document has no root element.");
            }

            DOMSource domSource = null;

            if (schemaElement.getLocalName().equals("import")) {

                String location = schemaElement.getAttribute("schemaLocation");

                // if the location is undefined or begins with #, then
                // import SHOULD be referencing an already deployed schema
                // component, i.e. if reference is broken, a deployment
                // exception will thrown by referencing component when it
                // is deployed

                if (location.equals("") || location.startsWith("#")) {
                    continue;
                }

                String baseURI = idlDocument.getDocumentURI();
                String expandedLocation;
                try {
                    expandedLocation = null;
                    if (baseURI == null) {
                        expandedLocation = new URI(location).toString();
                    } else {
                        expandedLocation = new URI(baseURI).resolve(location).toString();
                    }
                } catch (URISyntaxException ue) {
                    throw new DeploymentException("Error resolving imported document." + ue);
                }

                // definition is defined remotely. allow underlying provider to
                // dereference the location which may involve using optional
                // feature ws-mex

                Thread thread = Thread.currentThread();
                ClassLoader cl = thread.getContextClassLoader();
                InputSource is = null;
                try {
                    thread.setContextClassLoader(factoryContext.getClassLoader());
                    is = ((Provider)provider).spi_dereferenceLocation(componentContext, expandedLocation);
                } catch (Exception ex) {
                    throw new DeploymentException("Error attempting to resolve imported document at location '"
                            + location
                            + "'. "
                            + ex);
                } finally {
                    thread.setContextClassLoader(cl);
                }

                SAXSource saxSource = new SAXSource(is);
                saxSource.setSystemId(location);

                Document document = null;
                try {
                    document = new DocumentParser().parse(componentContext, is);
                } catch (Exception ex) {
                    throw new DeploymentException("Error attempting to parse imported document at location '"
                            + location
                            + "'. "
                            + ex);
                }

                domSource = new DOMSource(document.getDocumentElement());
                domSource.setSystemId(expandedLocation);

            } else {

                // the schema defintion is inlined

                domSource = new DOMSource(schemaElement);
                domSource.setSystemId(idlDocument.getDocumentURI());

            }

            Set<SchemaDocument> runtimeProviders = readProviders(componentContext, domSource);

            // iterate over the returned providers and each to component deployment.
            // then create a corresponding schema component for each and add it to
            // component deployment as well

            for (SchemaDocument schemaDocument : runtimeProviders) {
                Schema schema = null;
                if (schemaDocument.getTargetNamespace().equals(XMLSchema.TYPE)) {
                    schema = buildBuiltInSchema();
                } else {
                    boolean isWSDL11 = idlDocument.getExtensionType().equals("http://schemas.xmlsoap.org/wsdl/");
                    schema = buildSchema(deployment, schemaDocument, isWSDL11);
                }
                deployment.addComponent(schema);
                deployment.addProvider(schemaDocument);
            }

        }

    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.rt.ProviderReader#readProviders(org.bluestemsoftware.specification.eoa.component.ComponentContext,
     *      javax.xml.transform.dom.DOMSource)
     */
    @SuppressWarnings("unchecked")
    public Set<SchemaDocument> readProviders(ComponentContext componentContext, DOMSource source) throws DeploymentException {

        Element schemaElement = (Element)source.getNode();

        String targetNamespace = schemaElement.getAttribute("targetNamespace");

        Set<SchemaDocument> runtimeProviders = new HashSet<SchemaDocument>();

        // see note on schema doc within eoa-specification artifact regarding
        // parsing schema for schemas

        if (targetNamespace.equals(XMLSchema.TYPE)) {
            runtimeProviders.add(new BuiltInSchemaDocument(this));
            return runtimeProviders;
        }

        // serialize the source such that provider can parse into a set of
        // schema documents. note that we set classloader that parsed the
        // schema element, which is either wsdl factory deployment loader
        // or system classloader, depending upon how provider definition
        // was sourced, to context classsloader, i.e. because lsserializer
        // may use context classloader to load some classes

        Thread thread = Thread.currentThread();
        ClassLoader cl = thread.getContextClassLoader();
        InputSource is = null;
        try {
            thread.setContextClassLoader(schemaElement.getClass().getClassLoader());
            is = new InputSource(source.getSystemId());
            try {
                DOMImplementation di = schemaElement.getOwnerDocument().getImplementation();
                DOMImplementationLS ls = (DOMImplementationLS)di.getFeature("LS", "3.0");
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                OutputStreamWriter sw = new OutputStreamWriter(baos);
                LSSerializer lsSerializer = ls.createLSSerializer();
                LSOutput destination = ls.createLSOutput();
                destination.setCharacterStream(sw);
                destination.setEncoding("UTF-8");
                lsSerializer.write(schemaElement, destination);
                is.setByteStream(new ByteArrayInputStream(baos.toByteArray()));
                is.setEncoding("UTF-8");
            } catch (Exception ex) {
                throw new DeploymentException("Error reading domSource. " + ex);
            }
        } finally {
            thread.setContextClassLoader(cl);
        }

        Set<SchemaDocument> providers = null;
        try {
            providers = readSchemas(componentContext, new SAXSource(is));
        } catch (XMLSchemaException xe) {
            throw new DeploymentException(xe.getMessage());
        }

        // compiler complains set of xmlschemas can't be cast to a set
        // of runtime root components. hmmm ...
        return Set.class.cast(providers);

    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.ext.schema.SchemaFactory#buildSchema(org.bluestemsoftware.specification.eoa.component.ComponentDeployment,
     *      org.bluestemsoftware.specification.eoa.component.types.rt.SchemaDocument)
     */
    @Override
    public Schema buildSchema(ComponentDeployment deployment, SchemaDocument provider) throws DeploymentException {
        if (provider.getTargetNamespace().equals(XMLSchema.TYPE)) {
            return buildBuiltInSchema();
        } else {
            return buildSchema(deployment, provider, false);
        }
    }

    private Schema buildSchema(ComponentDeployment deployment, SchemaDocument provider, boolean isWSDL11) throws DeploymentException {

        XMLSchemaElement schemaElement = (XMLSchemaElement)provider.getDocumentElement();
        String targetNamespace = schemaElement.getTargetNamespace();
        String schemaID = schemaElement.getID();
        ComponentName name = new ComponentName(targetNamespace, schemaID);

        Map<SchemaComponent.Type, Map<String, SchemaComponent>> schemaComponents = null;
        schemaComponents = new HashMap<SchemaComponent.Type, Map<String, SchemaComponent>>();
        Map<String, SchemaComponent> elementDecls = new HashMap<String, SchemaComponent>();
        schemaComponents.put(SchemaComponent.Type.ELEMENT_DECLARATION, elementDecls);
        Map<String, SchemaComponent> typeDefs = new HashMap<String, SchemaComponent>();
        schemaComponents.put(SchemaComponent.Type.TYPE_DEFINITION, typeDefs);

        for (XSElementDeclaration declElement : schemaElement.getElementDeclarations()) {
            String declName = declElement.getName();
            Map<QName, SchemaFragment> fragments = getFragments(schemaElement, declElement);
            boolean isMessagePayload = false;
            if (isWSDL11) {
                // do not create implied abstract message if schema sourced from a wsdl 1.1 def
            } else {
                isMessagePayload = createImpliedAbstractMessage(deployment, targetNamespace, declElement);
            }
            elementDecls.put(declName, new ElementDeclaration(declName, fragments, isMessagePayload));
        }
        
        for (XSTypeDefinition typeDefinition : schemaElement.getTypeDefinitions()) {
            String declName = typeDefinition.getName();
            Map<QName, SchemaFragment> fragments = getFragments(schemaElement, typeDefinition);
            if (typeDefinition instanceof XSSimpleTypeDefinition) {
                typeDefs.put(declName, new TypeDefinition(declName, fragments, true));
            } else {
                typeDefs.put(declName, new TypeDefinition(declName, fragments, false));
            }
        }

        // retrieve schema reference elements, i.e. imports and includes. note
        // that redefines are not supported. underlying provider should log a
        // warning if redefine elements found while parsing document

        Map<String, Set<SchemaImport>> schemaImports = new HashMap<String, Set<SchemaImport>>();
        Set<SchemaInclude> schemaIncludes = new HashSet<SchemaInclude>();
        for (XMLSchemaReference refElement : schemaElement.getSchemaReferences()) {
            FragmentIdentifier schemaFragmentID = refElement.getSchemaFragmentIdentifier();
            String tns = schemaFragmentID.getRootComponentNamespace();
            String sid = schemaFragmentID.getPointerPart();
            ComponentName sname = new ComponentName(tns, sid);
            String location = refElement.getSchemaLocation();
            if (refElement instanceof XMLSchemaImport) {
                String importedNamespace = ((XMLSchemaImport)refElement).getNamespace();
                Set<SchemaImport> set = schemaImports.get(importedNamespace);
                if (set == null) {
                    set = new HashSet<SchemaImport>();
                    schemaImports.put(importedNamespace, set);
                }
                set.add(new SchemaImport(sname, location, importedNamespace));
            }
            if (refElement instanceof XMLSchemaInclude) {
                schemaIncludes.add(new SchemaInclude(sname, location));
            }
        }

        Map<String, SchemaProperty> properties = getProperties(schemaElement);

        return new Schema(name, schemaComponents, properties, schemaImports, schemaIncludes);

    }

    private Element findAppInfoElement(Element parent) {
        Element annotation = getChildElement(parent, new QName(XMLSchema.TYPE, "annotation"));
        if (annotation == null) {
            return null;
        }
        Element appinfo = getChildElement(annotation, new QName(XMLSchema.TYPE, "appinfo"));
        if (appinfo == null) {
            return null;
        }
        if (appinfo.getAttribute("source").equals("http://bluestemsoftware.org/specification/eoa/1.0")) {
            return appinfo;
        } else {
            return null;
        }
    }

    @SuppressWarnings("unchecked")
    private Map<QName, SchemaFragment> getFragments(XMLSchemaElement schemaElement, Element annotated) throws DeploymentException {

        Map<QName, SchemaFragment> fragments = new HashMap<QName, SchemaFragment>();

        Element appinfo = findAppInfoElement(annotated);

        if (appinfo == null) {
            return fragments;
        }

        String targetNamespace = schemaElement.getTargetNamespace();

        // parse contents of app info into a fragment component. note that query
        // MAY contain prefixed names which must resolve to a namespace decl on an
        // enclosing element. we cannot know which ns mappings are required without
        // parsing the query, and ... because the language is extensible we do not
        // know how, or if, qnames are serialized. so ... we grab all decls in scope

        NodeList nodeList = appinfo.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            if (nodeList.item(i).getNodeType() != Node.ELEMENT_NODE) {
                continue;
            }
            Element fragmentElement = (Element)nodeList.item(i);
            if (!fragmentElement.getLocalName().equals("fragment")) {
                continue;
            }
            String attrib = fragmentElement.getAttribute("property");
            if (attrib.equals("")) {
                continue;
            }
            QName propertyName = ComponentName.valueOf(fragmentElement, attrib);
            if (fragments.containsKey(propertyName)) {
                continue;
            }
            if (!propertyName.getNamespaceURI().equals(targetNamespace)) {
                if (!isNamespaceImported(schemaElement, propertyName.getNamespaceURI())) {
                    throw new DeploymentException("Property namespace "
                            + propertyName.getNamespaceURI()
                            + " referenced by fragment MUST be imported.");
                }
            }
            Element queryElement = getChildElement(fragmentElement, QUERY_ELEMENT);
            if (queryElement == null) {
                continue;
            }
            String queryLanguage = queryElement.getAttribute("language");
            queryLanguage = queryLanguage.equals("") ? XPathExpression.TYPE : queryLanguage;
            String query = getText(queryElement);
            Map<String, String> namespaces = new HashMap<String, String>();
            Node parent = queryElement;
            while (parent != null && parent.getNodeType() == Node.ELEMENT_NODE) {
                NamedNodeMap attributes = parent.getAttributes();
                for (int x = 0; x < attributes.getLength(); x++) {
                    Attr attr = (Attr)attributes.item(x);
                    if (attr.getPrefix() != null && attr.getPrefix().equals("xmlns")) {
                        String prefix = attr.getLocalName();
                        String namespace = attr.getValue();
                        if (namespaces.get(prefix) == null) {
                            namespaces.put(prefix, namespace);
                        }
                    }
                }
                parent = parent.getParentNode();
            }

            ExpressionRT runtimeExpression = null;
            ExpressionFactory ef = SystemContext.getContext().getSystem().getExpressionFactory(queryLanguage);
            if (ef == null) {
                throw new DeploymentException("ExpressionFactory of type " + queryLanguage + " is undefined.");
            } else {
                try {
                    runtimeExpression = ef.createExpression(namespaces, query);
                } catch (ExpressionException xe) {
                    throw new DeploymentException("Error creating runtime expression of type "
                            + queryLanguage
                            + " and query "
                            + query
                            + ". "
                            + xe);
                }
            }

            FragmentExpression fe = new FragmentExpression(query, namespaces, runtimeExpression);

            fragments.put(propertyName, new SchemaFragment(propertyName.getNamespaceURI(), propertyName
                    .getLocalPart(), fe));

        }

        return fragments;

    }

    private Map<String, SchemaProperty> getProperties(XMLSchemaElement schemaElement) throws DeploymentException {

        Map<String, SchemaProperty> properties = new HashMap<String, SchemaProperty>();

        Element appinfo = findAppInfoElement(schemaElement);

        if (appinfo == null) {
            return properties;
        }

        String targetNamespace = schemaElement.getTargetNamespace();

        NodeList nodeList = appinfo.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            if (nodeList.item(i).getNodeType() != Node.ELEMENT_NODE) {
                continue;
            }
            Element propertyElement = (Element)nodeList.item(i);
            if (!propertyElement.getLocalName().equals("property")) {
                continue;
            }

            String name = propertyElement.getAttribute("name");
            if (properties.containsKey(name)) {
                continue;
            }
            String type = propertyElement.getAttribute("type");
            String element = propertyElement.getAttribute("element");
            SchemaComponent.Type componentType = null;
            QName componentName = null;
            if (element.equals("")) {
                componentType = SchemaComponent.Type.TYPE_DEFINITION;
                componentName = ComponentName.valueOf(propertyElement, type);
            } else {
                componentType = SchemaComponent.Type.ELEMENT_DECLARATION;
                componentName = ComponentName.valueOf(propertyElement, element);
            }
            if (!componentName.getNamespaceURI().equals(targetNamespace)) {
                if (!isNamespaceImported(schemaElement, componentName.getNamespaceURI())) {
                    throw new DeploymentException("Component namespace "
                            + componentName.getNamespaceURI()
                            + " referenced by property '"
                            + name
                            + "' MUST be imported.");
                }
            }
            properties.put(name, new SchemaProperty(name, componentName.getNamespaceURI(), componentType,
                    componentName.getLocalPart()));
        }

        return properties;

    }

    private boolean isNamespaceImported(XMLSchemaElement schemaElement, String namespace) {
        if (namespace.equals(XMLSchema.TYPE)) {
            return true; // the import is implied
        }
        for (XMLSchemaReference schemaReference : schemaElement.getSchemaReferences()) {
            if (schemaReference instanceof XMLSchemaImport) {
                String importedNamespace = ((XMLSchemaImport)schemaReference).getNamespace();
                if (importedNamespace.equals(namespace)) {
                    return true;
                }
            }
        }
        return false;
    }

    private Schema buildBuiltInSchema() {

        Map<String, Set<SchemaImport>> imports = new HashMap<String, Set<SchemaImport>>();
        Set<SchemaInclude> includes = new HashSet<SchemaInclude>();
        Map<String, SchemaProperty> properties = new HashMap<String, SchemaProperty>();
        Map<QName, SchemaFragment> fragments = new HashMap<QName, SchemaFragment>();
        Map<SchemaComponent.Type, Map<String, SchemaComponent>> schemaComponents;
        schemaComponents = new HashMap<SchemaComponent.Type, Map<String, SchemaComponent>>();
        Map<String, SchemaComponent> elementDecls = new HashMap<String, SchemaComponent>();
        schemaComponents.put(SchemaComponent.Type.ELEMENT_DECLARATION, elementDecls);
        Map<String, SchemaComponent> typeDefs = new HashMap<String, SchemaComponent>();
        schemaComponents.put(SchemaComponent.Type.TYPE_DEFINITION, typeDefs);

        typeDefs.put("anyType", new TypeDefinition("string", fragments, false));
        typeDefs.put("string", new TypeDefinition("string", fragments, true));
        typeDefs.put("boolean", new TypeDefinition("boolean", fragments, true));
        typeDefs.put("decimal", new TypeDefinition("decimal", fragments, true));
        typeDefs.put("float", new TypeDefinition("float", fragments, true));
        typeDefs.put("double", new TypeDefinition("double", fragments, true));
        typeDefs.put("duration", new TypeDefinition("duration", fragments, true));
        typeDefs.put("dateTime", new TypeDefinition("dateTime", fragments, true));
        typeDefs.put("time", new TypeDefinition("time", fragments, true));
        typeDefs.put("date", new TypeDefinition("date", fragments, true));
        typeDefs.put("gYearMonth", new TypeDefinition("gYearMonth", fragments, true));
        typeDefs.put("gYear", new TypeDefinition("gYear", fragments, true));
        typeDefs.put("gMonthDay", new TypeDefinition("gMonthDay", fragments, true));
        typeDefs.put("gDay", new TypeDefinition("gDay", fragments, true));
        typeDefs.put("gMonth", new TypeDefinition("gMonth", fragments, true));
        typeDefs.put("hexBinary", new TypeDefinition("hexBinary", fragments, true));
        typeDefs.put("base64Binary", new TypeDefinition("base64Binary", fragments, true));
        typeDefs.put("anyURI", new TypeDefinition("anyURI", fragments, true));
        typeDefs.put("QName", new TypeDefinition("QName", fragments, true));
        typeDefs.put("NOTATION", new TypeDefinition("NOTATION", fragments, true));
        typeDefs.put("normalizedString", new TypeDefinition("normalizedString", fragments, true));
        typeDefs.put("token", new TypeDefinition("token", fragments, true));
        typeDefs.put("language", new TypeDefinition("language", fragments, true));
        typeDefs.put("NMTOKEN", new TypeDefinition("NMTOKEN", fragments, true));
        typeDefs.put("NMTOKENS", new TypeDefinition("NMTOKENS", fragments, true));
        typeDefs.put("Name", new TypeDefinition("Name", fragments, true));
        typeDefs.put("NCName", new TypeDefinition("NCName", fragments, true));
        typeDefs.put("ID", new TypeDefinition("ID", fragments, true));
        typeDefs.put("IDREF", new TypeDefinition("IDREF", fragments, true));
        typeDefs.put("IDREFS", new TypeDefinition("IDREFS", fragments, true));
        typeDefs.put("ENTITY", new TypeDefinition("ENTITY", fragments, true));
        typeDefs.put("ENTITIES", new TypeDefinition("ENTITIES", fragments, true));
        typeDefs.put("integer", new TypeDefinition("integer", fragments, true));
        typeDefs.put("nonPositiveInteger", new TypeDefinition("nonPositiveInteger", fragments, true));
        typeDefs.put("negativeInteger", new TypeDefinition("negativeInteger", fragments, true));
        typeDefs.put("long", new TypeDefinition("long", fragments, true));
        typeDefs.put("int", new TypeDefinition("int", fragments, true));
        typeDefs.put("short", new TypeDefinition("short", fragments, true));
        typeDefs.put("byte", new TypeDefinition("byte", fragments, true));
        typeDefs.put("nonNegativeInteger", new TypeDefinition("nonNegativeInteger", fragments, true));
        typeDefs.put("unsignedLong", new TypeDefinition("unsignedLong", fragments, true));
        typeDefs.put("unsignedInt", new TypeDefinition("unsignedInt", fragments, true));
        typeDefs.put("unsignedShort", new TypeDefinition("unsignedShort", fragments, true));
        typeDefs.put("unsignedByte", new TypeDefinition("unsignedByte", fragments, true));
        typeDefs.put("positiveInteger", new TypeDefinition("positiveInteger", fragments, true));

        // add wildcard declaration which will be used to type payload
        // of built-in message AnyMessage

        elementDecls.put("any", new ElementDeclaration("any", fragments, true));

        return new Schema(new ComponentName(XMLSchema.TYPE, ""), schemaComponents, properties, imports, includes);

    }

    private String getText(Element element) {
        if (element == null || !element.hasChildNodes()) {
            return null;
        }
        try {
            Text textNode = (Text)element.getFirstChild();
            return textNode.getData();
        } catch (ClassCastException ce) {
        }
        return null;
    }
    
    private Element getChildElement(Element parent, QName childName) {
        NodeList nodeList = parent.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            if (nodeList.item(i).getNodeType() != Node.ELEMENT_NODE) {
                continue;
            }
            Element element = (Element)nodeList.item(i);
            if (childName.getNamespaceURI().equals(XMLConstants.NULL_NS_URI)) {
                if (element.getNamespaceURI() != null) {
                    continue;
                }
            } else {
                if (element.getNamespaceURI() == null) {
                    continue;
                }
                if (!childName.getNamespaceURI().equals("*")
                        && !element.getNamespaceURI().equals(childName.getNamespaceURI())) {
                    continue;
                }
            }
            if (!childName.getLocalPart().equals("*") && !element.getLocalName().equals(childName.getLocalPart())) {
                continue;
            }
            return element;
        }
        return null;
    }

    private static class BuiltInSchemaDocument implements XMLSchema {

        public static final ComponentName NAME = new ComponentName(XMLSchema.TYPE, "");
        public static final FragmentIdentifier FID = new FragmentIdentifier(XMLSchema.TYPE, Scheme.EOA,
                "schemaDocument", "");

        private XMLSchemaFactory factory = null;
        private ComponentContext componentContext;

        public BuiltInSchemaDocument(XMLSchemaFactory factory) {
            this.factory = factory;
        }

        public Document clone(String documentURI) throws SchemaException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node adoptNode(Node source) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node appendChild(Node newChild) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node cloneNode(boolean deep) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public short compareDocumentPosition(Node other) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Attr createAttribute(String name) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Attr createAttributeNS(String namespaceURI, String qualifiedName) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public CDATASection createCDATASection(String data) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Comment createComment(String data) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public DocumentFragment createDocumentFragment() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Element createElement(String tagName) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Element createElementNS(String namespaceURI, String qualifiedName) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public EntityReference createEntityReference(String name) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public ProcessingInstruction createProcessingInstruction(String target, String data) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Text createTextNode(String data) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public NamedNodeMap getAttributes() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String getBaseURI() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public NodeList getChildNodes() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public DocumentType getDoctype() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Element getDocumentElement() {
            return null;
        }

        public String getDocumentURI() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public DOMConfiguration getDomConfig() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Element getElementById(String elementId) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public NodeList getElementsByTagName(String tagname) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public NodeList getElementsByTagNameNS(String namespaceURI, String localName) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Object getFeature(String feature, String version) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node getFirstChild() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public DOMImplementation getImplementation() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String getInputEncoding() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node getLastChild() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String getLocalName() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String getNamespaceURI() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node getNextSibling() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String getNodeName() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public short getNodeType() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String getNodeValue() throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Document getOwnerDocument() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node getParentNode() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String getPrefix() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node getPreviousSibling() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public RuntimeProvidable getRootComponent() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public boolean getStrictErrorChecking() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String getTargetNamespace() {
            return XMLSchema.TYPE;
        }

        public String getTextContent() throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Object getUserData(String key) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String getXmlEncoding() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public boolean getXmlStandalone() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String getXmlVersion() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public boolean hasAttributes() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public boolean hasChildNodes() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node importNode(Node importedNode, boolean deep) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node insertBefore(Node newChild, Node refChild) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public boolean isDefaultNamespace(String namespaceURI) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public boolean isEqualNode(Node arg) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public boolean isSameNode(Node other) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public boolean isSupported(String feature, String version) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String lookupNamespaceURI(String prefix) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String lookupPrefix(String namespaceURI) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public void normalize() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public void normalizeDocument() {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node removeChild(Node oldChild) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node renameNode(Node n, String namespaceURI, String qualifiedName) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Node replaceChild(Node newChild, Node oldChild) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public void setDocumentURI(String documentURI) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public void setNodeValue(String nodeValue) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public void setPrefix(String prefix) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public void setStrictErrorChecking(boolean strictErrorChecking) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public void setTextContent(String textContent) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Object setUserData(String key, Object data, UserDataHandler handler) {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public void setXmlStandalone(boolean xmlStandalone) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public void setXmlVersion(String xmlVersion) throws DOMException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public void validate(Element content) throws SchemaException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public String getExtensionType() {
            return XMLSchema.TYPE;
        }

        public ExtensionFactory getExtensionFactory() {
            return factory;
        }

        public org.bluestemsoftware.specification.eoa.ext.Extension.Provider getExtensionProvider() {
            return null;
        }

        public void deploy(ComponentContext componentContext) throws DeploymentException {
            this.componentContext = componentContext;
        }

        public ComponentContext getComponentContext() {
            return componentContext;
        }

        public ComponentType getComponentType() {
            return ComponentType.SCHEMA;
        }

        public ComponentName getName() {
            return NAME;
        }

        public RuntimeProvidable getRuntimeProvidable() {
            return (Schema)SystemContext.getContext().getSystem().getComponent(FID);
        }

        public boolean isDeployed() {
            return true;
        }

        public FragmentIdentifier getFragmentIdentifier() {
            return FID;
        }

        public void validate(Element content, ContentHandler contentHandler, LexicalHandler lexicalHandler, ErrorHandler errorHandler) throws SchemaException {
            throw new UnsupportedOperationException("Operation not supported on XML schema for schemas document.");
        }

        public Map<String, Style> modelStyles(InterfaceMessageReference interfaceMessageReference) throws SchemaException {
            return null;
        }

    }

    /*
     * Creates a WSDL 20 style message for each G.E.D. defined within a schema scoped to
     * System.
     */
    private boolean createImpliedAbstractMessage(ComponentDeployment deployment, String tns, Element elementDecl) {

        // we create the definition unless element decl is annotated with a message
        // extension element flagged otherwise

        boolean createMessageDefinition = true;

        Element appInfo = findAppInfoElement(elementDecl);
        if (appInfo != null) {
            Element messageElement = getChildElement(appInfo, MESSAGE_ELEMENT);
            if (messageElement != null) {
                String temp = messageElement.getAttribute("isMessagePayload");
                createMessageDefinition = Boolean.parseBoolean(temp);
            }
        }

        if (!createMessageDefinition) {
            return false;
        }

        // payload type serves as proxy for wsdl:message name
        String declName = elementDecl.getAttribute("name");
        ComponentName messageName = new ComponentName(tns, declName);

        // create a message component with a single part named 'payload'
        // which references GED
        String partName = MessagePart.PAYLOAD;
        Map<String, MessagePart> messageParts = new HashMap<String, MessagePart>();
        MessagePart part = new MessagePart(partName, tns, SchemaComponent.Type.ELEMENT_DECLARATION, declName);
        messageParts.put(MessagePart.PAYLOAD, part);

        // no attachment point exists for wsdl:message within wsdl 20, so we
        // pass an empty attachment
        ElementPolicy elementPolicy = null;

        // create abstract message component and add to deployment
        InterfaceMessage im = new InterfaceMessage(messageName, elementPolicy, messageParts);
        deployment.addComponent(im);

        return true;

    }

}
